from io import StringIO as io
import unittest

from nacc import redcap2nacc

headers = "ptid,redcap_event_name,redcap_data_access_group,formver,adcid,visitmo,visitday,visityr,visitnum,initials,header_complete,eng_preferred_language,eng_years_speak_spanish,eng_years_speak_english,eng_percentage_spanish,eng_percentage_english,eng_proficiency_spanish,eng_proficiency_read_spanish,eng_proficiency_write_spanish,eng_proficiency_oral_spanish,eng_proficiency_speak_english,eng_proficiency_read_english,eng_proficiency_write_english,eng_proficiency_oral_english,form_cls_linguistic_history_of_subject_complete,a2sub,a2not,a2comm,a3sub,a3not,a3comm,a4sub,a4not,a4comm,b1sub,b1not,b1comm,b5sub,b5not,b5comm,b6sub,b6not,b6comm,b7sub,b7not,b7comm,ivp_z1_complete,instructions_z1_9faf8a,a1lang,a2lang,a2sub_095a3b,a2not_21e87d,a2comm_391782,a3lang,a3sub_2b0d69,a3not_c7cb57,a3comm_ce8d68,a4lang,a4sub_2c437c,a4not_c4e53e,a4comm_278514,a5lang,b1lang,b1sub_3c9b3b,b1not_8b7733,b1comm_9d31a8,b4lang,b5lang,b5sub_712f66,b5not_a4b779,b5comm_ecb8a1,b6lang,b6sub_35db4c,b6not_06dff0,b6comm_8fe6dd,b7lang,b7sub_7e2220,b7not_2dfac5,b7comm_48c561,b8lang,b9lang,c2lang,d1lang,d2lang,ftld_present,a3alang,a3asubmitted,a3anot,b3flang,b9flang,c1flang,c2flang,c3flang,c4flang,c4fsubmitted,c4fnot,c5fsubmitted,c5fnot,c6fsubmitted,c6fnot,e2flang,e3flang,clslang,clssubmitted,ivp_z1x_complete,reason,refersc,learned,prestat,prespart,source,birthmo,birthyr,sex,hispanic,hispor,hisporx,race,racex,racesec,racesecx,raceter,raceterx,primlang,primlanx,educ,maristat,livsitua,independ,residenc,zip,handed,ivp_a1_complete,inbirmo,inbiryr,insex,inhisp,inhispor,inhispox,inrace,inracex,inrasec,inrasecx,inrater,inraterx,ineduc,inrelto,inknown,inlivwth,invisits,incalls,inrely,ivp_a2_complete,afffamm,fadmut,fadmutx,fadmuso,fadmusox,fftdmut,fftdmutx,fftdmuso,fftdmusx,fothmut,fothmutx,fothmuso,fothmusx,mommob,momyob,momdage,momneur,momprdx,mommoe,momageo,dadmob,dadyob,daddage,dadneur,dadprdx,dadmoe,dadageo,sibs,sib1mob,sib1yob,sib1agd,sib1neu,sib1pdx,sib1moe,sib1ago,sib2mob,sib2yob,sib2agd,sib2neu,sib2pdx,sib2moe,sib2ago,sib3mob,sib3yob,sib3agd,sib3neu,sib3pdx,sib3moe,sib3ago,sib4mob,sib4yob,sib4agd,sib4neu,sib4pdx,sib4moe,sib4ago,sib5mob,sib5yob,sib5agd,sib5neu,sib5pdx,sib5moe,sib5ago,sib6mob,sib6yob,sib6agd,sib6neu,sib6pdx,sib6moe,sib6ago,sib7mob,sib7yob,sib7agd,sib7neu,sib7pdx,sib7moe,sib7ago,sib8mob,sib8yob,sib8agd,sib8neu,sib8pdx,sib8moe,sib8ago,sib9mob,sib9yob,sib9agd,sib9neu,sib9pdx,sib9moe,sib9ago,sib10mob,sib10yob,sib10agd,sib10neu,sib10pdx,sib10moe,sib10ago,sib11mob,sib11yob,sib11agd,sib11neu,sib11pdx,sib11moe,sib11ago,sib12mob,sib12yob,sib12agd,sib12neu,sib12pdx,sib12moe,sib12ago,sib13mob,sib13yob,sib13agd,sib13neu,sib13pdx,sib13moe,sib13ago,sib14mob,sib14yob,sib14agd,sib14neu,sib14pdx,sib14moe,sib14ago,sib15mob,sib15yob,sib15agd,sib15neu,sib15pdx,sib15moe,sib15ago,sib16mob,sib16yob,sib16agd,sib16neu,sib16pdx,sib16moe,sib16ago,sib17mob,sib17yob,sib17agd,sib17neu,sib17pdx,sib17moe,sib17ago,sib18mob,sib18yob,sib18agd,sib18neu,sib18pdx,sib18moe,sib18ago,sib19mob,sib19yob,sib19agd,sib19neu,sib19pdx,sib19moe,sib19ago,sib20mob,sib20yob,sib20agd,sib20neu,sib20pdx,sib20moe,sib20ago,kids,kid1mob,kid1yob,kid1agd,kid1neu,kid1pdx,kid1moe,kid1ago,kid2mob,kid2yob,kid2agd,kid2neu,kid2pdx,kid2moe,kid2ago,kid3mob,kid3yob,kid3agd,kid3neu,kid3pdx,kid3moe,kid3ago,kid4mob,kid4yob,kid4agd,kid4neu,kid4pdx,kid4moe,kid4ago,kid5mob,kid5yob,kid5agd,kid5neu,kid5pdx,kid5moe,kid5ago,kid6mob,kid6yob,kid6agd,kid6neu,kid6pdx,kid6moe,kid6ago,kid7mob,kid7yob,kid7agd,kid7neu,kid7pdx,kid7moe,kid7ago,kid8mob,kid8yob,kid8agd,kid8neu,kid8pdx,kid8moe,kid8ago,kid9mob,kid9yob,kid9agd,kid9neu,kid9pdx,kid9moe,kid9ago,kid10mob,kid10yob,kid10agd,kid10neu,kid10pdx,kid10moe,kid10ago,kid11mob,kid11yob,kid11agd,kid11neu,kid11pdx,kid11moe,kid11ago,kid12mob,kid12yob,kid12agd,kid12neu,kid12pdx,kid12moe,kid12ago,kid13mob,kid13yob,kid13agd,kid13neu,kid13pdx,kid13moe,kid13ago,kid14mob,kid14yob,kid14agd,kid14neu,kid14pdx,kid14moe,kid14ago,kid15mob,kid15yob,kid15agd,kid15neu,kid15pdx,kid15moe,kid15ago,ivp_a3_complete,anymeds,drugid_1,drugid_2,drugid_3,drugid_4,drugid_5,drugid_6,drugid_7,drugid_8,drugid_9,drugid_10,drugid_11,drugid_12,drugid_13,drugid_14,drugid_15,drugid_16,drugid_17,drugid_18,drugid_19,drugid_20,drugid_21,drugid_22,drugid_23,drugid_24,drugid_25,drugid_26,drugid_27,drugid_28,drugid_29,drugid_30,drugid_31,drug_name_31,drugid_32,drug_name_32,drugid_33,drug_name_33,drugid_34,drug_name_34,drugid_35,drug_name_35,drugid_36,drug_name_36,drugid_37,drug_name_37,drugid_38,drug_name_38,drugid_39,drug_name_39,drugid_40,drug_name_40,drugid_41,drug_name_41,drugid_42,drug_name_42,drugid_43,drug_name_43,drugid_44,drug_name_44,drugid_45,drug_name_45,drugid_46,drug_name_46,drugid_47,drug_name_47,drugid_48,drug_name_48,drugid_49,drug_name_49,drugid_50,drug_name_50,ivp_a4_complete,tobac30,tobac100,smokyrs,packsper,quitsmok,alcoccas,alcfreq,cvhatt,hattmult,hattyear,cvafib,cvangio,cvbypass,cvpacdef,cvchf,cvangina,cvhvalve,cvothr,cvothrx,cbstroke,strokmul,strokyr,cbtia,tiamult,tiayear,pd,pdyr,pdothr,pdothryr,seizures,tbi,tbibrief,tbiexten,tbiwolos,tbiyear,diabetes,diabtype,hyperten,hypercho,b12def,thyroid,arthrit,arthtype,arthtypx,arthupex,arthloex,arthspin,arthunk,incontu,incontf,apnea,rbd,insomn,othsleep,othsleex,alcohol,abusothr,abusx,ptsd,bipolar,schiz,dep2yrs,depothr,anxiety,ocd,npsydev,psycdis,psycdisx,ivp_a5_complete,height,weight,bpsys,bpdias,hrate,vision,viscorr,viswcorr,hearing,hearaid,hearwaid,ivp_b1_complete,b2_1_abrupt,b2_2_stepwise,b2_3_somatic,b2_4_emot,b2_5_hxhyper,b2_6_hxstroke,b2_7_foclsym,b2_8_foclsign,b2_9_hachin,b2_10_cvdcog,b2_11_strokcog,b2_12_cvdimag,b2_12a_1_cvdimag1,b2_12a_2_cvdimag2,b2_12a_3_cvdimag3,b2_12a_4_cvdimag4,b2_12a_5_cvdimagx,b2_evaluation_form_his_and_cvd_complete,b3_g1_pdnormal___0,b3_1_speech,b3_1a_speechx,b3_2_facexp,b3_2a_facexpx,b3_3a_trestfac,b3_3a1_trestfax,b3_3b_trestrhd,b3_3b1_trestrhx,b3_3c_trestlhd,b3_3c1_trestlhx,b3_3d_trestrft,b3_3d1_trestrfx,b3_3e_trestlft,b3_3e1_trestlfx,b3_4a_tractrhd,b3_4a1_tractrhx,b3_4b_tractlhd,b3_4b1_tractlhx,b3_5a_rigdneck,b3_5a1_rigdnex,b3_5b_rigduprt,b3_5b1_rigduprx,b3_5c_rigduplf,b3_5c1_rigduplx,b3_5d_rigdlort,b3_5d1_rigdlorx,b3_5e_rigdlolf,b3_5e1_rigdlolx,b3_6a_tapsrt,b3_6a1_tapsrtx,b3_6b_tapslf,b3_6b1_tapslfx,b3_7a_handmovr,b3_7a1_handmvrx,b3_7b_handmovl,b3_7b1_handmvlx,b3_8a_handaltr,b3_8a1_handatrx,b3_8b_handaltl,b3_8b1_handatlx,b3_9a_legrt,b3_9a1_legrtx,b3_9b_leglf,b3_9b1_leglfx,b3_10_arising,b3_10a_arisingx,b3_11_posture,b3_11a_posturex,b3_12_gait,b3_12a_gaitx,b3_13_posstab,b3_13a_posstabx,b3_14_bradykin,b3_14a_bradykix,b3_15_total,b3_evaluation_form_updrs_complete,memory,orient,judgment,commun,homehobb,perscare,cdrsum,cdrglob,comport,cdrlang,ivp_b4_complete,npiqinf,npiqinfx,del,delsev,hall,hallsev,agit,agitsev,depd,depdsev,anx,anxsev,elat,elatsev,apa,apasev,disn,disnsev,irr,irrsev,mot,motsev,nite,nitesev,app,appsev,ivp_b5_complete,nogds,satis,dropact,empty,bored,spirits,afraid,happy,helpless,stayhome,memprob,wondrful,wrthless,energy,hopeless,better,gds,ivp_b6_complete,bills,taxes,shopping,games,stove,mealprep,events,payattn,remdates,travel,ivp_b7_complete,normexam,parksign,resttrl,slowingl,rigidl,resttrr,slowingr,rigidr,brady,parkgait,postinst,cvdsigns,cortdef,sivdfind,cvdmotl,cortvisl,somatl,cvdmotr,cortvisr,somatr,postcort,pspcbs,eyepsp,dyspsp,axialpsp,gaitpsp,apraxsp,apraxl,cortsenl,ataxl,alienlml,dystonl,myocllt,apraxr,cortsenr,ataxr,alienlmr,dystonr,myoclrt,alsfind,gaitnph,otherneur,otherneurx,ivp_b8_complete,decsub,decin,decclcog,cogmem,cogori,cogjudg,coglang,cogvis,cogattn,cogfluc,cogflago,cogothr,cogothrx,cogfpred,cogfprex,cogmode,cogmodex,decage,decclbe,beapathy,bedep,bevhall,bevwell,bevhago,beahall,bedel,bedisin,beirrit,beagit,beperch,berem,beremago,beanx,beothr,beothrx,befpred,befpredx,bemode,bemodex,beage,decclmot,mogait,mofalls,motrem,moslow,mofrst,momode,momodex,momopark,parkage,momoals,alsage,moage,course,frstchg,lbdeval,ftldeval,ivp_b9_complete,c1s_1a_mmseloc,c1s_1a1_mmselan,c1s_1a2_mmselanx,c1s_1b1_mmseorda,c1s_1b2_mmseorlo,c1s_1c_pentagon,c1s_1d_mmse,c1s_2_npsycloc,c1s_2a_npsylan,c1s_2a1_npsylanx,c1s_3amo_logimo,c1s_3ady_logiday,c1s_3ayr_logiyr,c1s_3a1_logiprev,c1s_3b_logimem,c1s_4a_digif,c1s_4b_digiflen,c1s_5a_digib,c1s_5b_digiblen,c1s_6a_animals,c1s_6b_veg,c1s_7a_traila,c1s_7a1_trailarr,c1s_7a2_trailali,c1s_7b_trailb,c1s_7b1_trailbrr,c1s_7b2_trailbli,c1s_8a_wais,c1s_9a_memunits,c1s_9b_memtime,c1s_10a_boston,c1s_11a_cogstat,ivp_c1s_complete,mocacomp,mocareas,mocaloc,mocalan,mocalanx,mocavis,mocahear,mocatots,mocatrai,mocacube,mocacloc,mocaclon,mocacloh,mocanami,mocaregi,mocadigi,mocalett,mocaser7,mocarepe,mocaflue,mocaabst,mocarecn,mocarecc,mocarecr,mocaordt,mocaormo,mocaoryr,mocaordy,mocaorpl,mocaorct,npsycloc_c2,npsylan_c2,npsylanx_c2,craftvrs,crafturs,udsbentc,digforct,digforsl,digbacct,digbacls,animals_c2,veg_c2,traila_c2,trailarr_c2,trailali_c2,trailb_c2,trailbrr_c2,trailbli_c2,craftdvr,craftdre,craftdti,craftcue,udsbentd,udsbenrs,minttots,minttotw,mintscng,mintscnc,mintpcng,mintpcnc,udsverfc,udsverfn,udsvernf,udsverlc,udsverlr,udsverln,udsvertn,udsverte,udsverti,cogstat_c2,ivp_c2_complete,dxmethod,normcog,demented,amndem,pca,ppasyn,ppasynt,ftdsyn,lbdsyn,namndem,mciamem,mciaplus,mciaplan,mciapatt,mciapex,mciapvis,mcinon1,mcin1lan,mcin1att,mcin1ex,mcin1vis,mcinon2,mcin2lan,mcin2att,mcin2ex,mcin2vis,impnomci,amylpet,amylcsf,fdgad,hippatr,taupetad,csftau,fdgftld,tpetftld,mrftld,datscan,othbiom,othbiomx,imaglinf,imaglac,imagmach,imagmich,imagmwmh,imagewmh,admut,ftldmut,othmut,othmutx,alzdis,alzdisif,lbdis,lbdif,park,msa,msaif,psp,pspif,cort,cortif,ftldmo,ftldmoif,ftldnos,ftldnoif,ftldsubt,ftldsubx,cvd,cvdif,prevstk,strokedec,stkimag,infnetw,infwmh,esstrem,esstreif,downs,downsif,hunt,huntif,prion,prionif,brninj,brninjif,brnincte,hyceph,hycephif,epilep,epilepif,neop,neopif,neopstat,hiv,hivif,othcog,othcogif,othcogx,dep,depif,deptreat,bipoldx,bipoldif,schizop,schizoif,anxiet,anxietif,delir,delirif,ptsddx,ptsddxif,othpsy,othpsyif,othpsyx,alcdem,alcdemif,alcabuse,impsub,impsubif,dysill,dysillif,meds,medsif,cogoth,cogothif,cogothx,cogoth2,cogoth2f,cogoth2x,cogoth3,cogoth3f,cogoth3x,ivp_d1_complete,cancer,cancsite,diabet,myoinf,conghrt,afibrill,hypert,angina,hypchol,vb12def,thydis,arth,artype,artypex,artupex,artloex,artspin,artunkn,urineinc,bowlinc,sleepap,remdis,hyposom,sleepoth,sleepotx,angiocp,angiopci,pacemake,hvalve,antienc,antiencx,othcond,othcondx,ivp_d2_complete,dob,datedraw,gender,freezeroptions,shelfoptions,rackoptions,box,numaliquots,buffycoatoptions,plasmaoptions,serumoptions,mt_sinai_samples_complete,npformmo,npformdy,npformyr,npid,npsex,npdage,npdodmo,npdoddy,npdodyr,nppmih,npfix,npfixx,npwbrwt,npwbrf,npgrcca,npgrla,npgrha,npgrsnh,npgrlch,npavas,nptan,nptanx,npaban,npabanx,npasan,npasanx,nptdpan,nptdpanx,nphismb,nphisg,nphisss,nphist,nphiso,nphisox,npthal,npbraak,npneur,npadnc,npdiff,npamy,npinf,npinf1a,npinf1b,npinf1d,npinf1f,npinf2a,npinf2b,npinf2d,npinf2f,npinf3a,npinf3b,npinf3d,npinf3f,npinf4a,npinf4b,npinf4d,npinf4f,nphemo,nphemo1,nphemo2,nphemo3,npold,npold1,npold2,npold3,npold4,npoldd,npoldd1,npoldd2,npoldd3,npoldd4,nparter,npwmr,nppath,npnec,nppath2,nppath3,nppath4,nppath5,nppath6,nppath7,nppath8,nppath9,nppath10,nppath11,nppatho,nppathox,nplbod,npnloss,nphipscl,nptdpa,nptdpb,nptdpc,nptdpd,nptdpe,npftdtau,nppick,npftdt2,npcort,npprog,npftdt5,npftdt6,npftdt7,npftdt8,npftdt9,npftdt10,npftdtdp,npalsmnd,npoftd,npoftd1,npoftd2,npoftd3,npoftd4,npoftd5,nppdxa,nppdxb,nppdxc,nppdxd,nppdxe,nppdxf,nppdxg,nppdxh,nppdxi,nppdxj,nppdxk,nppdxl,nppdxm,nppdxn,nppdxo,nppdxp,nppdxq,nppdxr,nppdxrx,nppdxs,nppdxsx,nppdxt,nppdxtx,npbnka,npbnkb,npbnkc,npbnkd,npbnke,npbnkf,npbnkg,npfaut,npfaut1,npfaut2,npfaut3,npfaut4,neuropathology_complete,fu_a2sub,fu_a2not,fu_a2comm,fu_a3sub,fu_a3not,fu_a3comm,fu_a4sub,fu_a4not,fu_a4comm,fu_b1sub,fu_b1not,fu_b1comm,fu_b5sub,fu_b5not,fu_b5comm,fu_b6sub,fu_b6not,fu_b6comm,fu_b7sub,fu_b7not,fu_b7comm,fvp_z1_complete,fu_instructions,fu_a1lang,fu_a2lang,fu_a2sub_73fdc7,fu_a2not_fd65a7,fu_a2comm_e20905,fu_a3lang,fu_a3sub_c2a68b,fu_a3not_f7c411,fu_a3comm_455b3e,fu_a4lang,fu_a4sub_143f22,fu_a4not_b95e64,fu_a4comm_374c44,fu_b1lang,fu_b1sub_c03500,fu_b1not_0a7e9f,fu_b1comm_b4e94e,fu_b4lang,fu_b5lang,fu_b5sub_51a694,b5not_fvpz1x,fu_b5comm_2e7bc8,fu_b6lang,fu_b6sub_db439d,fu_b6not_310244,fu_b6comm_587d18,fu_b7lang,fu_b7sub_21a95f,fu_b7not_dccb30,fu_b7comm_0cb804,fu_b8lang,fu_b9lang,fu_c1lang,fu_c2lang,fu_d1lang,fu_d2lang,fu_ftld_present,fu_a3alang,fu_a3asubmitted,fu_a3anot,fu_b3flang,fu_b9flang,fu_c1flang,fu_c2flang,fu_c3flang,fu_c4flang,fu_c4fsubmitted,fu_c4fnot,fu_c5fsubmitted,fu_c5fnot,fu_c6fsubmitted,fu_c6fnot,fu_e2flang,fu_e3flang,fu_clslang,fu_clssubmitted,fvp_z1x_complete,fu_birthmo,fu_birthyr,fu_maristat,fu_sex,fu_livsitua,fu_independ,fu_residenc,fu_zip,fvp_a1_complete,fu_inbirmo,fu_inbiryr,fu_insex,fu_newinf,fu_inhisp,fu_inhispor,fu_inhispox,fu_inrace,fu_inracex,fu_inrasec,fu_inrasecx,fu_inrater,fu_inraterx,fu_ineduc,fu_inrelto,fu_inknown,fu_inlivwth,fu_invisits,fu_incalls,fu_inrely,fvp_a2_complete,fu_nwinfmut,fu_fadmut,fu_fadmutx,fu_fadmuso,fu_fadmusox,fu_fftdmut,fu_fftdmutx,fu_fftdmuso,fu_fftdmusx,fu_fothmut,fu_fothmutx,fu_fothmuso,fu_fothmusx,fu_nwinfpar,fu_mommob,fu_momyob,fu_momdage,fu_momneur,fu_momprdx,fu_mommoe,fu_momageo,fu_dadmob,fu_dadyob,fu_daddage,fu_dadneur,fu_dadprdx,fu_dadmoe,fu_dadageo,fu_sibs,fu_nwinfsib,fu_sib1mob,fu_sib1yob,fu_sib1agd,fu_sib1neu,fu_sib1pdx,fu_sib1moe,fu_sib1ago,fu_sib2mob,fu_sib2yob,fu_sib2agd,fu_sib2neu,fu_sib2pdx,fu_sib2moe,fu_sib2ago,fu_sib3mob,fu_sib3yob,fu_sib3agd,fu_sib3neu,fu_sib3pdx,fu_sib3moe,fu_sib3ago,fu_sib4mob,fu_sib4yob,fu_sib4agd,fu_sib4neu,fu_sib4pdx,fu_sib4moe,fu_sib4ago,fu_sib5mob,fu_sib5yob,fu_sib5agd,fu_sib5neu,fu_sib5pdx,fu_sib5moe,fu_sib5ago,fu_sib6mob,fu_sib6yob,fu_sib6agd,fu_sib6neu,fu_sib6pdx,fu_sib6moe,fu_sib6ago,fu_sib7mob,fu_sib7yob,fu_sib7agd,fu_sib7neu,fu_sib7pdx,fu_sib7moe,fu_sib7ago,fu_sib8mob,fu_sib8yob,fu_sib8agd,fu_sib8neu,fu_sib8pdx,fu_sib8moe,fu_sib8ago,fu_sib9mob,fu_sib9yob,fu_sib9agd,fu_sib9neu,fu_sib9pdx,fu_sib9moe,fu_sib9ago,fu_sib10mob,fu_sib10yob,fu_sib10agd,fu_sib10neu,fu_sib10pdx,fu_sib10moe,fu_sib10ago,fu_sib11mob,fu_sib11yob,fu_sib11agd,fu_sib11neu,fu_sib11pdx,fu_sib11moe,fu_sib11ago,fu_sib12mob,fu_sib12yob,fu_sib12agd,fu_sib12neu,fu_sib12pdx,fu_sib12moe,fu_sib12ago,fu_sib13mob,fu_sib13yob,fu_sib13agd,fu_sib13neu,fu_sib13pdx,fu_sib13moe,fu_sib13ago,fu_sib14mob,fu_sib14yob,fu_sib14agd,fu_sib14neu,fu_sib14pdx,fu_sib14moe,fu_sib14ago,fu_sib15mob,fu_sib15yob,fu_sib15agd,fu_sib15neu,fu_sib15pdx,fu_sib15moe,fu_sib15ago,fu_sib16mob,fu_sib16yob,fu_sib16agd,fu_sib16neu,fu_sib16pdx,fu_sib16moe,fu_sib16ago,fu_sib17mob,fu_sib17yob,fu_sib17agd,fu_sib17neu,fusib17pdx,fu_sib17moe,fu_sib17ago,fu_sib18mob,fu_sib18yob,fu_sib18agd,fu_sib18neu,fu_sib18pdx,fu_sib18moe,fu_sib18ago,fu_sib19mob,fu_sib19yob,fu_sib19agd,fu_sib19neu,fu_sib19pdx,fu_sib19moe,fu_sib19ago,fu_sib20mob,fu_sib20yob,fu_sib20agd,fu_sib20neu,fu_sib20pdx,fu_sib20moe,fu_sib20ago,fu_kids,fu_nwinfkid,fu_kid1mob,fu_kid1yob,fu_kid1agd,fu_kid1neu,fu_kid1pdx,fu_kid1moe,fu_kid1ago,fu_kid2mob,fu_kid2yob,fu_kid2agd,fu_kid2neu,fu_kid2pdx,fu_kid2moe,fu_kid2ago,fu_kid3mob,fu_kid3yob,fu_kid3agd,fu_kid3neu,fu_kid3pdx,fu_kid3moe,fu_kid3ago,fu_kid4mob,fu_kid4yob,fu_kid4agd,fu_kid4neu,fu_kid4pdx,fu_kid4moe,fu_kid4ago,fu_kid5mob,fu_kid5yob,fu_kid5agd,fu_kid5neu,fu_kid5pdx,fu_kid5moe,fu_kid5ago,fu_kid6mob,fu_kid6yob,fu_kid6agd,fu_kid6neu,fu_kid6pdx,fu_kid6moe,fu_kid6ago,fu_kid7mob,fu_kid7yob,fu_kid7agd,fu_kid7neu,fu_kid7pdx,fu_kid7moe,fu_kid7ago,fu_kid8mob,fu_kid8yob,fu_kid8agd,fu_kid8neu,fu_kid8pdx,fu_kid8moe,fu_kid8ago,fu_kid9mob,fu_kid9yob,fukid9agd,fu_kid9neu,fu_kid9pdx,fu_kid9moe,fu_kid9ago,fu_kid10mob,fu_kid10yob,fu_kid10agd,fu_kid10neu,fu_kid10pdx,fu_kid10moe,fu_kid10ago,fu_kid11mob,fu_kid11yob,fu_kid11agd,fu_kid11neu,fu_kid11pdx,fu_kid11moe,fu_kid11ago,fu_kid12mob,fu_kid12yob,fu_kid12agd,fu_kid12neu,fu_kid12pdx,fu_kid12moe,fu_kid12ago,fu_kid13mob,fu_kid13yob,fu_kid13agd,fu_kid13neu,fu_kid13pdx,fu_kid13moe,fu_kid13ago,fu_kid14mob,fu_kid14yob,fu_kid14agd,fu_kid14neu,fu_kid14pdx,fu_kid14moe,fu_kid14ago,fu_kid15mob,fu_kid15yob,fu_kid15agd,fu_kid15neu,fu_kid15pdx,fu_kid15moe,fu_kid15ago,fvp_a3_complete,fu_anymeds,fu_drugid_1,fu_drugid_2,fu_drugid_3,fu_drugid_4,fu_drugid_5,fu_drugid_6,fu_drugid_7,fu_drugid_8,fu_drugid_9,fu_drugid_10,fu_drugid_11,fu_drugid_12,fu_drugid_13,fu_drugid_14,fu_drugid_15,fu_drugid_16,fu_drugid_17,fu_drugid_18,fu_drugid_19,fu_drugid_20,fu_drugid_21,fu_drugid_22,fu_drugid_23,fu_drugid_24,fu_drugid_25,fu_drugid_26,fu_drugid_27,fu_drugid_28,fu_drugid_29,fu_drugid_30,fu_drugid_31,fu_drug_name_31,fu_drugid_32,fu_drug_name_32,fu_drugid_33,fu_drug_name_33,fu_drugid_34,fu_drug_name_34,fu_drugid_35,fu_drug_name_35,fu_drugid_36,fu_drug_name_36,fu_drugid_37,fu_drug_name_37,fu_drugid_38,fu_drug_name_38,fu_drugid_39,fu_drug_name_39,fu_drugid_40,fu_drug_name_40,fu_drugid_41,fu_drug_name_41,fu_drugid_42,fu_drug_name_42,fu_drugid_43,fu_drug_name_43,fu_drugid_44,fu_drug_name_44,fu_drugid_45,fu_drug_name_45,fu_drugid_46,fu_drug_name_46,fu_drugid_47,fu_drug_name_47,fu_drugid_48,fu_drug_name_48,fu_drugid_49,fu_drug_name_49,fu_drugid_50,fu_drug_name_50,fvp_a4_complete,fu_height,fu_weight,fu_bpsys,fu_bpdias,fu_hrate,fu_vision,fu_viscorr,fu_viswcorr,fu_hearing,fu_hearaid,fu_hearwaid,fvp_b1_complete,fu_memory,fu_orient,fu_judgment,fu_commun,fu_homehobb,fu_perscare,fu_cdrsum,fu_cdrglob,fu_comport,fu_cdrlang,fvp_b4_complete,fu_npiqinf,fu_npiqinfx,fu_del,fu_delsev,fu_hall,fu_hallsev,fu_agit,fu_agitsev,fu_depd,fu_depdsev,fu_anx,fu_anxsev,fu_elat,fu_elatsev,fu_apa,fu_apasev,fu_disn,fu_disnsev,fu_irr,fu_irrsev,fu_mot,fu_motsev,fu_nite,fu_nitesev,fu_app,fu_appsev,fvp_b5_complete,fu_nogds,fu_satis,fu_dropact,fu_empty,fu_bored,fu_spirits,fu_afraid,fu_happy,fu_helpless,fu_stayhome,fu_memprob,fu_wondrful,fu_wrthless,fu_energy,fu_hopeless,fu_better,fu_gds,fvp_b6_complete,fu_bills,fu_taxes,fu_shopping,fu_games,fu_stove,fu_mealprep,fu_events,fu_payattn,fu_remdates,fu_travel,fvp_b7_complete,fu_normexam,fu_parksign,fu_resttrl,fu_slowingl,fu_rigidl,fu_resttrr,fu_slowingr,fu_rigidr,fu_brady,fu_parkgait,fu_postinst,fu_cvdsigns,fu_cortdef,fu_sivdfind,fu_cvdmotl,fu_cortvisl,fu_somatl,fu_cvdmotr,fu_cortvisr,fu_somatr,fu_postcort,fu_pspcbs,fu_eyepsp,fu_dyspsp,fu_axialpsp,fu_gaitpsp,fu_apraxsp,fu_apraxl,fu_cortsenl,fu_ataxl,fu_alienlml,fu_dystonl,fu_myocllt,fu_apraxr,fu_cortsenr,fu_ataxr,fu_alienlmr,fu_dystonr,fu_myoclrt,fu_alsfind,fu_gaitnph,fu_othneur,fu_othneurxs,fvp_b8_complete,fu_decsub,fu_decin,fu_decclcog,fu_cogmem,fu_cogori,fu_cogjudg,fu_coglang,fu_cogvis,fu_cogattn,fu_cogfluc,fu_cogflago,fu_cogothr,fu_cogothrx,fu_cogfpred,fu_cogfprex,fu_cogmode,fu_cogmodex,fu_decage,fu_decclbe,fu_beapathy,fu_bedep,fu_bevhall,fu_bevwell,fu_bevhago,fu_beahall,fu_bedel,fu_bedisin,fu_beirrit,fu_beagit,fu_beperch,fu_berem,fu_beremago,fu_beanx,fu_beothr,fu_beothrx,fu_befpred,fu_befpredx,fu_bemode,fu_bemodex,fu_beage,fu_decclmot,fu_mogait,fu_mofalls,fu_motrem,fu_moslow,fu_mofrst,fu_momode,fu_momodex,fu_momopark,fu_parkage,fu_momoals,fu_alsage,fu_moage,fu_course,fu_frstchg,fu_lbdeval,fu_ftldeval,fvp_b9_complete,fu_c1s_1a_mmseloc,fu_c1s_1a1_mmselan,fu_c1s_1a2_mmselanx,fu_c1s_1b1_mmseorda,fu_c1s_1b2_mmseorlo,fu_c1s_1c_pentagon,fu_c1s_1d_mmse,fu_c1s_2_npsycloc,fu_c1s_2a_npsylan,fu_c1s_2a1_npsylanx,fu_c1s_3amo_logimo,fu_c1s_3ady_logiday,fu_c1s_3ayr_logiyr,fu_c1s_3a1_logiprev,fu_c1s_3b_logimem,fu_c1s_4a_digif,fu_c1s_4b_digiflen,fu_c1s_5a_digib,fu_c1s_5b_digiblen,fu_c1s_6a_animals,fu_c1s_6b_veg,fu_c1s_7a_traila,fu_c1s_7a1_trailarr,fu_c1s_7a2_trailali,fu_c1s_7b_trailb,fu_c1s_7b1_trailbrr,fu_c1s_7b2_trailbli,fu_c1s_8a_wais,fu_c1s_9a_memunits,fu_c1s_9b_memtime,fu_c1s_10a_boston,fu_c1s_11a_cogstat,fvp_c1s_complete,fu_mmsecomp,fu_mmsereas,fu_mmseloc,fu_mmselan,fu_mmselanx,fu_mmsevis,fu_mmsehear,fu_mmseorda,fu_mmseorlo,fu_pentagon,fu_mmse,fu_npsycloc,fu_npsylan,fu_npsylanx,fu_logimo,fu_logiday,fu_logiyr,fu_logiprev,fu_logimem,fu_udsbentc_c1,fu_digif,fu_digiflen,fu_digib,fu_digiblen,fu_animals,fu_veg,fu_traila,fu_trailarr,fu_trailali,fu_trailb,fu_trailbrr,fu_trailbli,fu_memunits,fu_memtime,fu_udsbentd_c1,fu_udsbenrs_c1,fu_boston,fu_udsverfc_c1,fu_udsverfn_c1,fu_udsvernf_c1,fu_udsverlc_c1,fu_udsverlr_c1,fu_udsverln_c1,fu_udsvertn_c1,fu_udsverte_c1,fu_udsverti_c1,fu_cogstat,fvp_c1_complete,fu_mocacomp,fu_mocareas,fu_mocaloc,fu_mocalan,fu_mocalanx,fu_mocavis,fu_mocahear,fu_mocatots,fu_mocatrai,fu_mocacube,fu_mocacloc,fu_mocaclon,fu_mocacloh,fu_mocanami,fu_mocaregi,fu_mocadigi,fu_mocalett,fu_mocaser7,fu_mocarepe,fu_mocaflue,fu_mocaabst,fu_mocarecn,fu_mocarecc,fu_mocarecr,fu_mocaordt,fu_mocaormo,fu_mocaoryr,fu_mocaordy,fu_mocaorpl,fu_mocaorct,fu_npsycloc_c2,fu_npsylan_c2,fu_npsylanx_c2,fu_craftvrs,fu_crafturs,fu_udsbentc,fu_digforct,fu_digforsl,fu_digbacct,fu_digbacls,fu_animals_c2,fu_veg_c2,fu_traila_c2,fu_trailarr_c2,fu_trailali_c2,fu_trailb_c2,fu_trailbrr_c2,fu_trailbli_c2,fu_craftdvr,fu_craftdre,fu_craftdti,fu_craftcue,fu_udsbentd,fu_udsbenrs,fu_minttots,fu_minttotw,fu_mintscng,fu_mintscnc,fu_mintpcng,fu_mintpcnc,fu_udsverfc,fu_udsverfn,fu_udsvernf,fu_udsverlc,fu_udsverlr,fu_udsverln,fu_udsvertn,fu_udsverte,fu_udsverti,fu_cogstat_c2,fvp_c2_complete,fu_dxmethod,fu_normcog,fu_demented,fu_amndem,fu_pca,fu_ppasyn,fu_ppasynt,fu_ftdsyn,fu_lbdsyn,fu_namndem,fu_mciamem,fu_mciaplus,fu_mciaplan,fu_mciapatt,fu_mciapex,fu_mciapvis,fu_mcinon1,fu_mcin1lan,fu_mcin1att,fu_mcin1ex,fu_mcin1vis,fu_mcinon2,fu_mcin2lan,fu_mcin2att,fu_mcin2ex,fu_mcin2vis,fu_impnomci,fu_amylpet,fu_amylcsf,fu_fdgad,fu_hippatr,fu_taupetad,fu_csftau,fu_fdgftld,fu_tpetftld,fu_mrftld,fu_datscan,fu_othbiom,fu_othbiomx,fu_imaglinf,fu_imaglac,fu_imagmach,fu_imagmich,fu_imagmwmh,fu_imagewmh,fu_admut,fu_ftldmut,fu_othmut,fu_othmutx,fu_alzdis,fu_alzdisif,fu_lbdis,fu_lbdif,fu_park,fu_msa,fu_msaif,fu_psp,fu_pspif,fu_cort,fu_cortif,fu_ftldmo,fu_ftldmoif,fu_ftldnos,fu_ftldnoif,fu_ftldsubt,fu_ftldsubx,fu_cvd,fu_cvdif,fu_prevstk,fu_strokdec,fu_stkimag,fu_infnetw,fu_infwmh,fu_esstrem,fu_esstreif,fu_downs,fu_downsif,fu_hunt,fu_huntif,fu_prion,fu_prionif,fu_brninj,fu_brninjif,fu_brnincte,fu_hyceph,fu_hycephif,fu_epilep,fu_epilepif,fu_neop,fu_neopif,fu_neopstat,fu_hiv,fu_hivif,fu_othcog,fu_othcogif,fu_othcogx,fu_dep,fu_depif,fu_deptreat,fu_bipoldx,fu_bipoldif,fu_schizop,fu_schizoif,fu_anxiet,fu_anxietif,fu_delir,fu_delirif,fu_ptsddx,fu_ptsddxif,fu_othpsy,fu_othpsyif,fu_othpsyx,fu_alcdem,fu_alcdemif,fu_alcabuse,fu_impsub,fu_impsubif,fu_dysill,fu_dysillif,fu_meds,fu_medsif,fu_cogoth,fu_cogothif,fu_cogothx,fu_cogoth2,fu_cogoth2f,fu_cogoth2x,fu_cogoth3,fu_cogoth3f,fu_cogoth3x,fvp_d1_complete,fu_cancer,fu_cancsite,fu_diabet,fu_myoinf,fu_conghrt,fu_afibrill,fu_hypert,fu_angina,fu_hypchol,fu_vb12def,fu_thydis,fu_arth,fu_artype,fu_artypex,fu_artupex,fu_artloex,fu_artspin,fu_artunkn,fu_urineinc,fu_bowlinc,fu_sleepap,fu_remdis,fu_hyposom,fu_sleepoth,fu_sleepotx,fu_angiocp,fu_angiopci,fu_pacemake,fu_hvalve,fu_antienc,fu_antiencx,fu_othcond,fu_othcondx,fvp_d2_complete,as_date,as_1_newthings,as_2_interest,as_3_concerned,as_4_mucheffort,as_5_sometodo,as_6_plans,as_7_motivation,as_8_energy,as_9_tellyou,as_10_indiff,as_11_unconcerned,as_12_push,as_13_neither,as_14_apathetic,as_15_calctotal,apathy_scale_complete,psms_2_examiner,psms_3_visit,psms_4_date,psms_5_ratedby,psms_6_ratedbyrel,psms_a_toileting,psms_b_feeding,psms_c_dressing,psms_d_grooming,psms_e_physamb,psms_e1_physambas,psms_f_bathing,psms_g_total,physical_self_maintenance_scale_psms_complete,se_a_name,se_b_date,se_1_pleasant,se_2_nervous,se_3_satisfied,se_4_wishhappy,se_5_failure,se_6_rested,se_7_calm,se_8_piling,se_9_worry,se_10_happy,se_11_disturbing,se_12_lackconf,se_13_secure,se_14_decisions,se_15_inadequate,se_16_content,se_17_bothers,se_18_disappointment,se_19_steady,se_20_tension,self_evaluation_questionnaire_complete,adcs_initials,adcs_date,adcs_infoobtained,adcs_1_findbelong,adcs_1a_perform,adcs_2_selectclothes,adcs_2a_clothperform,adcs_3_phydress,adcs_sspg1,adcs_4_cleanroom,adcs_4a_cleanperform,adcs_5_balance,adcs_5a_balanceperf,adcs_6_write,adcs_6a_writecomp,adcs_7_laundry,adcs_7a_laundryperf,adcs_sspg2,adcs_8_appoint,adcs_8a_aware,adcs_9_usephone,adcs_9a_phoneperf,adcs_10_makemeal,adcs_10a_foodprep,adcs_sspg3,adcs_11_travel,adcs_11a_travelperf,adcs_12_curevents,adcs_12a_regnatinter,adcs_12b_famfrndnbr,adcs_12c_evathome,adcs_12d_converse,adcs_sspg4,adcs_13_readmore,adcs_13a_selectask,adcs_13b_talkonread,adcs_13c_talkaftrread,adcs_14_watchtv,adcs_14a_askprgm,adcs_14b_talkprgm,adcs_14c_talkprgminaday,adcs_sspg5,adcs_15_goshop,adcs_15a_selectcorrect,adcs_15b_payown,adcs_16_onown,adcs_16a_lefthomemin,adcs_16b_lefthomehour,adcs_16c_lefthourless,adcs_sspg6,adcs_17_useappl,adcs_17i_whichappl___0,adcs_17i_whichappl___1,adcs_17i_whichappl___2,adcs_17i_whichappl___3,adcs_17i_whichappl___4,adcs_17i_whichappl___5,adcs_17i_whichappl___6,adcs_17i_whichappl___7,adcs_17i_whichappl___8,adcs_17i_whichappl___9,adcs_17ii_otherappl,adcs_17a_appluseperf,adcs_sspg7,adcs_18_hobby,adcs_18i_whichhobby___0,adcs_18i_whichhobby___1,adcs_18i_whichhobby___2,adcs_18i_whichhobby___3,adcs_18i_whichhobby___4,adcs_18i_whichhobby___5,adcs_18i_whichhobby___6,adcs_18i_whichhobby___7,adcs_18i_whichhobby___8,adcs_18i_whichhobby___9,adcs_18i_whichhobby___10,adcs_18i_whichhobby___11,adcs_18i_whichhobby___12,adcs_18i_whichhobby___13,adcs_18ii_otherhobby,adcs_18a_hobbyperf,adcs_18b_losthobby,adcs_sspg8,adcs_adltotal,adcs_totaldontknow,adcs_19_drivecar,adcs_driveperf,adcs_20_takemed,adcs_medperf,adcs_21_complexact,adcs_reminders,adcs_22_initiate,adcs_23_complextasttime,adcs_24_extenuating,adcs_extenuexpln,adcs_mci_activities_of_daily_living_invent_complete,mhs_psychiatr,mhs_psychhapp,mhs_psychwhen,mhs_psychhowlong,mhs_psychres,mhs_emoprblm,mhs_emohapp,mhs_emowhen,mhs_emolast,mhs_emores,mhs_takemed,mhs_medwhat,mhs_medwhatfor,mhs_medwhen,mhs_medlast,mhs_medres,mhs_psychemergncy,mhs_psychemerghapp,mhs_psychemergwhen,mhs_psychemerglast,mhs_psychemergres,mhs_voices,mhs_whatvoices,mhs_voicesduemed,mhs_death,mhs_deadperson,mhs_cultural,mhs_voiceswhen,mhs_voiceslast,mhs_voicesinterf,mhs_voicessad,mhs_thoughkill,mhs_killhapp,mhs_attemptkill,mhs_ideation,mhs_fullofenergy,mhs_duetomed,mhs_fullofenergylong,mhs_energysxs___0,mhs_energysxs___1,mhs_energysxs___2,mhs_energysxs___3,mhs_energysxs___4,mhs_energysxs___5,mhs_energysxs___6,mhs_fullenergyinterf,mhs_nightmare,mhs_nightmrecoll,mhs_distdream,mhs_reoccur,mhs_distress,mhs_physiorxn,mhs_stimuli,mhs_aroused,mhs_sxslast,mhs_sxsinterf,mhs_strongfears,mhs_fearexcess,mhs_panicatk,mhs_avoid,mhs_fearinterf,mhs_fearlong,mhs_spells,mhs_abrupt___1,mhs_abrupt___2,mhs_abrupt___3,mhs_abrupt___4,mhs_abrupt___5,mhs_abrupt___6,mhs_abrupt___7,mhs_abrupt___8,mhs_abrupt___9,mhs_abrupt___10,mhs_abrupt___11,mhs_abrupt___12,mhs_abrupt___13,mhs_ocd,mhs_ocdduemed,mhs_ocdexcess,mhs_ocdinterf,mhs_alcohol,mhs_alcoholmore,mhs_concern,mhs_concernmore,mhs_treatmt,mhs_treatmtmore,mhs_drink,mhs_usedtodrink,mhs_drugs,mhs_usedtodrugs,mhs_stop,mhs_annoy,mhs_guilty,mhs_want,mhs_totalcage,mental_health_screen_complete,csc_b_examiner,csc_c_visit,csc_d_date,csc_1a_slurred,csc_1b_labored,csc_1c_wordfind,csc_1d_wordsub,csc_1e_mispronounced,csc_1f_comprehesnion,csc_1g_reading,csc_2a_facial,csc_2b_room,csc_2c_focus,csc_2d_clock,csc_2e_direction,csc_3a_simple,csc_3b_complex,csc_3c_car,csc_3d_objects,csc_3e_clothes,csc_4a_cingulate,csc_4b_dorsolateral,csc_4c_orbitobasal,cortical_symptom_checklist_complete,cfia_1_memory,cfia_2_repeat,cfia_3_misplace,cfia_4_reminders,cfia_5_appointments,cfia_6_racalling,cfia_7_driving,cfia_8_money,cfia_9_social,cfia_10_work,cfia_11_news,cfia_12_activities,cfia_13_disoriented,cfia_14_household,cfia_15_doctor,cfia_16_method,cfia_17_comments,cognitive_function_index_participant_complete,cfib_1_memory,cfib_2_repeat,cfib_3_misplace,cfib_4_reminders,cfib_5_appointments,cfib_6_racalling,cfib_7_driving,cfib_8_money,cfib_9_social,cfib_10_work,cfib_11_news,cfib_12_activities,cfib_13_disoriented,cfib_14_household,cfib_15_doctor,cfib_16_studypartnerid,cfib_17_method,cfib_18_comments,cognitive_function_index_study_partner_complete,fl4_examinar,fl4_visit,fl4_date,fl4_a_source,fl4_b_infrat,fl4_1a_events,fl4_1b_tasks,fl4_1c_names,fl4_1d_words,fl4_1e_possess,fl4_1f_responsib,fl4_1_mean,fl4_2a_events,fl4_2b_within,fl4_2c_outside,fl4_2d_day,fl4_2e_month,fl4_2f_year,fl4_2_mean,fl4_3a_recognize_err,fl4_3b_show_caution,fl4_3c_plan_errand,fl4_3d_make_change,fl4_3e_handle_fin,fl4_3f_judge_grocery,fl4_3_mean,fl4_4a_self_motiv,fl4_4b_personal_hyg,fl4_4c_act_impulsiv,fl4_4d_failed_events,fl4_4e_people_feel,fl4_4f_react_as_before,fl4_4_mean,fl4_5a_garbage,fl4_5b_coffee,fl4_5c_complex,fl4_5d_non_routine,fl4_5e_tv_programs,fl4_5f_crossword,fl4_5_mean,fl4_6a_urine,fl4_6b_bowels,fl4_6c_clothes,fl4_6d_shower,fl4_6e_groom,fl4_6f_utensils,fl4_6_mean,fl4_1_max,fl4_2_max,fl4_3_max,fl4_4_max,fl4_5_max,fl4_6_max,fl4_sum_of_means,fl4_sum_of_max,fl4_mean_of_means,fl4_mean_of_max,fl4_modified_cdr_rating_complete,fl4s_a_source,fl4s_b_infrat,fl4s_1a_events,fl4s_1b_tasks,fl4s_1c_names,fl4s_1d_words,fl4s_1e_possess,fl4s_1f_responsib,fl4s_1_mean,fl4s_2a_events,fl4s_2b_within,fl4s_2c_outside,fl4s_2d_day,fl4s_2e_month,fl4s_2f_year,fl4s_2_mean,fl4s_3a_recognize_err,fl4s_3b_show_caution,fl4s_3c_plan_errand,fl4s_3d_make_change,fl4s_3e_handle_fin,fl4s_3f_judge_grocery,fl4s_3_mean,fl4s_4a_self_motiv,fl4s_4b_personal_hyg,fl4s_4c_act_impulsiv,fl4s_4d_failed_events,fl4s_4e_people_feel,fl4s_4f_react_as_before,fl4s_4_mean,fl4s_5a_simple,fl4s_5b_complex,fl4s_5c_non_routine,fl4s_5d_tv,fl4s_5e_crossword,fl4s_5_mean,fl4s_6a_urine,fl4s_6b_bowels,fl4s_6c_clothes,fl4s_6d_shower,fl4s_6e_groom,fl4s_6f_utensils,fl4s_6_mean,fl4s_1_max,fl4s_2_max,fl4s_3_max,fl4s_4_max,fl4s_5_max,fl4s_6_max,fl4s_sum_of_means,fl4s_sum_of_max,fl4s_mean_of_means,fl4s_mean_of_max,fl4s_modified_cdr_rating_complete,fl9_b_examinar,fl9_c_visit,fl9_d_date,fl9_e_site,fl9_f_dob,fl9_g_age,fl9_h_education,fl9_i_gender,fl9_j_ethnicity,fl9_k_language,fl9_k_language_other,fl9_k_language_secondary,fl9_k_language_ot2_b24,fl9_l_handedness,fl9_mmse_a_orientation,fl9_mmse_b_3word,fl9_mmse_c_total,fl9_mmse_d_serial,fl9_wrat_a_letter,fl9_wrat_b_word,fl9_wrat_c_total,fl9_nacc_a_immediate,fl9_nacc_b_delayed,fl9_bvmt_a_trail1,fl9_bvmt_b_trail2,fl9_bvmt_c_trail3,fl9_bvmt_d_total,fl9_bvmt_e_learning,fl9_bvmt_f_delay,fl9_bvmt_g_percent,fl9_hvlt_a_total,fl9_hvlt_b_delayed,fl9_hvlt_c_retention,fl9_lassi_arecall1_raw,fl9_lassi_arecall1_no,fl9_lassi_acued1_raw,fl9_lassi_acued1_no,fl9_lassi_acued2_raw,fl9_lassi_acued2_no,fl9_lassi_brecall1_raw,fl9_lassi_brecall1_no,fl9_lassi_bcued1_raw,fl9_lassi_bcued1_no,fl9_lassi_bcued2_raw,fl9_lassi_bcued2_no,fl9_lassi_arecall2_raw,fl9_lassi_arecall2_no,fl9_lassi_acued3_raw,fl9_lassi_acued3_no,fl9_lassi_delayed_raw,fl9_lassi_delayed_no,fl9_visual_sample_ms,fl9_visual_sample_prop,fl9_visual_shapes_ms,fl9_visual_shapes_prop,fl9_visual_color_ms,fl9_visual_color_prop,fl9_trails_a_raw,fl9_trails_a_errors,fl9_trails_b_raw,fl9_trails_b_errors,fl9_nab,fl9_nacc,fl9_stroop_word,fl9_stroop_color,fl9_stroop_cw,fl9_stroop_inter,fl9_wais,fl9_cowat_f,fl9_cowat_a,fl9_cowat_s,fl9_cowat_animals,fl9_cowat_fruits,fl9_cowat_veg,fl9_gds,fl9_adcs_total,fl9_adcs_dont,fl9_instrumental_adls_score,fl9_anxiety,fl9_apathy,fl9_vt_1_path,fl9_vt_1_time,fl9_vt_2_path,fl9_vt_2_time,fl9_vt_3_path,fl9_vt_3_time,fl9_vt_4_path,fl9_vt_4_time,fl9_ivt_1_1_target,fl9_ivt_1_1_total,fl9_ivt_1_1_path,fl9_ivt_1_1_time,fl9_ivt_1_1_qaudrant,fl9_ivt_1_2_target,fl9_ivt_1_2_total,fl9_ivt_1_2_path,fl9_ivt_1_2_time,fl9_ivt_1_2_qaudrant,fl9_ivt_1_3_target,fl9_ivt_1_3_total,fl9_ivt_1_3_path,fl9_ivt_1_3_time,fl9_ivt_1_3_qaudrant,fl9_ivt_1_4_target,fl9_ivt_1_4_total,fl9_ivt_1_4_path,fl9_ivt_1_4_time,fl9_ivt_1_4_qaudrant,fl9_ivt_1_5_target,fl9_ivt_1_5_total,fl9_ivt_1_5_path,fl9_ivt_1_5_time,fl9_ivt_1_5_qaudrant,fl9_ivt_1_6_target,fl9_ivt_1_6_total,fl9_ivt_1_6_path,fl9_ivt_1_6_time,fl9_ivt_1_6_qaudrant,fl9_ivt_1_7_target,fl9_ivt_1_7_total,fl9_ivt_1_7_path,fl9_ivt_1_7_time,fl9_ivt_1_7_qaudrant,fl9_ivt_1_8_target,fl9_ivt_1_8_total,fl9_ivt_1_8_path,fl9_ivt_1_8_time,fl9_ivt_1_8_qaudrant,fl9_ivt_1_9_target,fl9_ivt_1_9_total,fl9_ivt_1_9_path,fl9_ivt_1_9_time,fl9_ivt_1_9_qaudrant,fl9_ivt_2_1_search,fl9_ivt_2_1_comments,fl9_ivt_2_2_search,fl9_ivt_2_2_comments,fl9_ivt_2_3_search,fl9_ivt_2_3_comments,fl9_ivt_2_4_search,fl9_ivt_2_4_comments,fl9_ivt_2_5_search,fl9_ivt_2_5_comments,fl9_ivt_2_6_search,fl9_ivt_2_6_comments,fl9_ivt_2_7_search,fl9_ivt_2_7_comments,fl9_ivt_2_8_search,fl9_ivt_2_8_comments,fl9_ivt_2_9_search,fl9_ivt_2_9_comments,fl9_optional_bnt,fl9_optional_fluency,fl9_optional_forwards,fl9_optional_backwards,fl9_mitsi_a_asoc1_raw,fl9_mitsi_a_asoc1_ms,fl9_mitsi_a_asoc1_no,fl9_mitsi_a_asoc2_raw,fl9_mitsi_a_asoc2_ms,fl9_mitsi_a_asoc2_no,fl9_mitsi_b_asoc1_raw,fl9_mitsi_b_asoc1_ms,fl9_mitsi_b_asoc1_no,fl9_mitsi_b_asoc2_raw,fl9_mitsi_b_asoc2_ms,fl9_mitsi_b_asoc2_no,fl9_mitsi_a_short_raw,fl9_mitsi_a_short_ms,fl9_mitsi_a_short_no,fl9_mitsi_a_long_raw,fl9_mitsi_a_long_ms,fl9_mitsi_a_long_no,fl9_neuropsychological_battery_complete,fl9s_b_examinar,fl9s_c_visit,fl9s_d_date,fl9s_e_site,fl9s_mmse_a_orientation,fl9s_mmse_b_3word,fl9s_mmse_c_total,fl9s_mmse_d_serial,fl9s_woodcock_a_correct,fl9s_woodcock_b_ae,fl9s_woodcock_c_ge,fl9s_nacc_a_immediate,fl9s_nacc_b_delayed,fl9s_bvmt_a_trial1,fl9s_bvmt_b_trial2,fl9s_bvmt_c_trial3,fl9s_bvmt_d_total,fl9s_bvmt_e_learning,fl9s_bvmt_f_delay,fl9s_bvmt_g_percent,fl9s_hvlt_a_total,fl9s_hvlt_b_delayed,fl9s_hvlt_c_retention,fl9s_lassi_arecall1_raw,fl9s_lassi_arecall1_no,fl9s_lassi_acued1_raw,fl9s_lassi_acued1_no,fl9s_lassi_acued2_raw,fl9s_lassi_acued2_no,fl9s_lassi_brecall1_raw,fl9s_lassi_brecall1_no,fl9s_lassi_bcued1_raw,fl9s_lassi_bcued1_no,fl9s_lassi_bcued2_raw,fl9s_lassi_bcued2_no,fl9s_lassi_arecall2_raw,fl9s_lassi_arecall2_no,fl9s_lassi_acued3_raw,fl9s_lassi_acued3_no,fl9s_lassi_delayed_raw,fl9s_lassi_delayed_no,fl9s_visual_sample_ms,fl9s_visual_sample_prop,fl9s_visual_shapes_ms,fl9s_visual_shapes_prop,fl9s_visual_color_ms,fl9s_visual_color_prop,fl9s_visual_total_duration,fl9s_trials_a_raw,fl9s_trials_a_errors,fl9s_trials_b_raw,fl9s_trials_b_errors,fl9s_nab,fl9s_nacc,fl9s_stroop_word,fl9s_stroop_word_ac,fl9s_stroop_color,fl9s_stroop_color_ac,fl9s_stroop_cw,fl9s_stroop_cw_ac,fl9s_stroop_inter,fl9s_wais,fl9s_cowat_f,fl9s_cowat_a,fl9s_cowat_s,fl9s_cowat_animals,fl9s_cowat_fruits,fl9s_cowat_veg,fl9s_gds,fl9s_adcs_total,fl9s_adcs_dont,fl9s_anxiety,fl9s_vt_1_path,fl9s_vt_1_time,fl9s_vt_2_path,fl9s_vt_2_time,fl9s_vt_3_path,fl9s_vt_3_time,fl9s_vt_4_path,fl9s_vt_4_time,fl9s_ivt_1_1_target,fl9s_ivt_1_1_total,fl9s_ivt_1_1_path,fl9s_ivt_1_1_time,fl9s_ivt_1_1_qaudrant,fl9s_ivt_1_2_target,fl9s_ivt_1_2_total,fl9s_ivt_1_2_path,fl9s_ivt_1_2_time,fl9s_ivt_1_2_qaudrant,fl9s_ivt_1_3_target,fl9s_ivt_1_3_total,fl9s_ivt_1_3_path,fl9s_ivt_1_3_time,fl9s_ivt_1_3_qaudrant,fl9s_ivt_1_4_target,fl9s_ivt_1_4_total,fl9s_ivt_1_4_path,fl9s_ivt_1_4_time,fl9s_ivt_1_4_qaudrant,fl9s_ivt_1_5_target,fl9s_ivt_1_5_total,fl9s_ivt_1_5_path,fl9s_ivt_1_5_time,fl9s_ivt_1_5_qaudrant,fl9s_ivt_1_6_target,fl9s_ivt_1_6_total,fl9s_ivt_1_6_path,fl9s_ivt_1_6_time,fl9s_ivt_1_6_qaudrant,fl9s_ivt_1_7_target,fl9s_ivt_1_7_total,fl9s_ivt_1_7_path,fl9s_ivt_1_7_time,fl9s_ivt_1_7_qaudrant,fl9s_ivt_1_8_target,fl9s_ivt_1_8_total,fl9s_ivt_1_8_path,fl9s_ivt_1_8_time,fl9s_ivt_1_8_qaudrant,fl9s_ivt_1_9_total,fl9s_ivt_1_9_qaudrant,fl9s_ivt_2_1_search,fl9s_ivt_2_1_comments,fl9s_ivt_2_2_search,fl9s_ivt_2_2_comments,fl9s_ivt_2_3_search,fl9s_ivt_2_3_comments,fl9s_ivt_2_4_search,fl9s_ivt_2_4_comments,fl9s_ivt_2_5_search,fl9s_ivt_2_5_comments,fl9s_ivt_2_6_search,fl9s_ivt_2_6_comments,fl9s_ivt_2_7_search,fl9s_ivt_2_7_comments,fl9s_ivt_2_8_search,fl9s_ivt_2_8_comments,fl9s_ivt_2_9_search,fl9s_ivt_2_9_comments,fl9s_apathy,fl9s_mitsi_a_asoc1_raw,fl9s_mitsi_a_asoc1_ms,fl9s_mitsi_a_asoc1_no,fl9s_mitsi_a_asoc2_raw,fl9s_mitsi_a_asoc2_ms,fl9s_mitsi_a_asoc2_no,fl9s_mitsi_b_asoc1_raw,fl9s_mitsi_b_asoc1_ms,fl9s_mitsi_b_asoc1_no,fl9s_mitsi_b_asoc2_raw,fl9s_mitsi_b_asoc2_ms,fl9s_mitsi_b_asoc2_no,fl9s_mitsi_a_short_raw,fl9s_mitsi_a_short_ms,fl9s_mitsi_a_short_no,fl9s_mitsi_a_long_raw,fl9s_mitsi_a_long_ms,fl9s_mitsi_a_long_no,fl9s_optional_bnt,fl9s_optional_fluency,fl9s_optional_forwards,fl9s_optional_backwards,fl9s_neuropsychological_battery_complete,diff_obj_crct,diff_obj_preexp_no_om,diff_obj_no_prexp,diff_obj_no_preexp_no_om,diff_obj_om,diff_obj_com,diff_ani_no_preexp,diff_non_obj_no_preexp,mdn_reactn_time_diff_obj,mdn_reactn_time_diff_ani,mdn_reactn_time_diff_non_obj,free_obj_recall,free_loc_recall,cor_hit_count,cor_crct_rejection_count,cor_false_reject_count,cor_false_recog_count,ola_hit_count,ola_avg_loc_score,ordt_opat_form_complete,consesusfinaldx,finalclindx,finalnpdx,ethnicity,ethnicity_other,age1,edu,fl11_nonnacc_complete,mri_rater,date_mri_acquisition,r_diffuse_cortical_atrophy,l_diffuse_cortical_atrophy,diffuse_cortical_atrophy_score,r_frontal_atrophy,l_frontal_atrophy,frontal_atrophy_score,r_temporal_atrohy,l_temporal_atrohy,temporal_atrohy_score,r_parietal_atrophy,l_parietal_atrophy,parietal_atrophy_score,r_lat_vent_enlarged,l_lat_vent_enlarged,lat_vent_enlarged_score,iii_vent_enlarged,iii_vent_enlarged_score,r_hpc_atrophy,l_hpc_atrophy,hpc_atrophy_score,r_erc_atrophy,l_erc_atrophy,erc_atrophy_score,r_large_vessel_infarcts,l_large_vessel_infarcts,large_vessel_infarcts_score,r_lacunar_infarcts,l_lacunar_infarcts,lacunar_infarcts_score,r_macrohemorrhage,l_macrohemorrhage,macrohemorrhage_score,r_microhemorrhage,l_microhemorrhage,microhemorrhage_score,r_wh_matter_hyperintensity,l_wh_matter_hyperintensity,wh_matter_hyperinten_score,r_other,l_other,other_score,other_detail,right_total_score,left_total_score,right_left_total_score,mri_features_complete,apoe,apoe_results_complete,m1_start,m1_form_date,m1_1,m1_2a,m1_2a1,m1_2b___1,m1_2b___2,m1_2b___3,m1_2b___4,m1_2b___5,m1_2b___6,m1_2b1,m1_3,m1_3a,m1_3a1,m1_4,m1_5a,m1_5b,m1_6a,m1_6b,ee5_research_structural_mri_complete,ndv_adcid,ndv_ptid,ndv_packet,ndv_formver,ndv_visitmo,ndv_visitday,ndv_visityr,ndv_naccvnum,ndv_naccavst,ndv_naccnvst,ndv_naccdays,ndv_naccfdys,ndv_naccreas,ndv_naccrefr,ndv_birthmo,ndv_birthyr,ndv_sex,ndv_hispanic,ndv_hispor,ndv_hisporx,ndv_race,ndv_racex,ndv_racesec,ndv_racesecx,ndv_raceter,ndv_raceterx,ndv_primlang,ndv_primlanx,ndv_educ,ndv_maristat,ndv_nacclivs,ndv_independ,ndv_residenc,ndv_handed,ndv_maristax,ndv_residenx,ndv_inbirmo,ndv_inbiryr,ndv_insex,ndv_newinf,ndv_inhisp,ndv_inhispor,ndv_inhispox,ndv_naccninr,ndv_inrace,ndv_inracex,ndv_inrasec,ndv_inrasecx,ndv_inrater,ndv_inraterx,ndv_ineduc,ndv_inrelto,ndv_inreltox,ndv_inknown,ndv_inlivwth,ndv_invisits,ndv_incalls,ndv_inrely,ndv_naccfam,ndv_naccmom,ndv_naccdad,ndv_naccam,ndv_naccamx,ndv_naccams,ndv_naccamsx,ndv_naccfm,ndv_naccfmx,ndv_naccfms,ndv_naccfmsx,ndv_naccom,ndv_naccomx,ndv_naccoms,ndv_naccomsx,ndv_naccfadm,ndv_naccfftd,ndv_anymeds,ndv_drug1,ndv_drug2,ndv_drug3,ndv_drug4,ndv_drug5,ndv_drug6,ndv_drug7,ndv_drug8,ndv_drug9,ndv_drug10,ndv_drug11,ndv_drug12,ndv_drug13,ndv_drug14,ndv_drug15,ndv_drug16,ndv_drug17,ndv_drug18,ndv_drug19,ndv_drug20,ndv_drug21,ndv_drug22,ndv_drug23,ndv_drug24,ndv_drug25,ndv_drug26,ndv_drug27,ndv_drug28,ndv_drug29,ndv_drug30,ndv_drug31,ndv_drug32,ndv_drug33,ndv_drug34,ndv_drug35,ndv_drug36,ndv_drug37,ndv_drug38,ndv_drug39,ndv_drug40,ndv_tobac30,ndv_tobac100,ndv_smokyrs,ndv_packsper,ndv_quitsmok,ndv_alcoccas,ndv_alcfreq,ndv_cvhatt,ndv_hattmult,ndv_hattyear,ndv_cvafib,ndv_cvangio,ndv_cvbypass,ndv_cvpacdef,ndv_cvpace,ndv_cvchf,ndv_cvangina,ndv_cvhvalve,ndv_cvothr,ndv_cvothrx,ndv_cbstroke,ndv_strokmul,ndv_naccstyr,ndv_cbtia,ndv_tiamult,ndv_nacctiyr,ndv_pd,ndv_pdyr,ndv_pdothr,ndv_pdothryr,ndv_seizures,ndv_nacctbi,ndv_tbi,ndv_tbibrief,ndv_traumbrf,ndv_tbiexten,ndv_traumext,ndv_tbiwolos,ndv_traumchr,ndv_tbiyear,ndv_ncothr,ndv_ncothrx,ndv_diabetes,ndv_diabtype,ndv_hyperten,ndv_hypercho,ndv_b12def,ndv_thyroid,ndv_arthrit,ndv_arthtype,ndv_arthtypx,ndv_arthupex,ndv_arthloex,ndv_arthspin,ndv_arthunk,ndv_incontu,ndv_incontf,ndv_apnea,ndv_rbd,ndv_insomn,ndv_othsleep,ndv_othsleex,ndv_alcohol,ndv_abusothr,ndv_abusx,ndv_ptsd,ndv_bipolar,ndv_schiz,ndv_dep2yrs,ndv_depothr,ndv_anxiety,ndv_ocd,ndv_npsydev,ndv_psycdis,ndv_psycdisx,ndv_height,ndv_weight,ndv_bpsys,ndv_bpdias,ndv_hrate,ndv_vision,ndv_viscorr,ndv_viswcorr,ndv_hearing,ndv_hearaid,ndv_hearwaid,ndv_abrupt,ndv_stepwise,ndv_somatic,ndv_emot,ndv_hxhyper,ndv_hxstroke,ndv_foclsym,ndv_foclsign,ndv_hachin,ndv_cvdcog,ndv_strokcog,ndv_cvdimag,ndv_cvdimag1,ndv_cvdimag2,ndv_cvdimag3,ndv_cvdimag4,ndv_cvdimagx,ndv_pdnormal,ndv_speech,ndv_speechx,ndv_facexp,ndv_facexpx,ndv_trestfac,ndv_trestfax,ndv_trestrhd,ndv_trestrhx,ndv_trestlhd,ndv_trestlhx,ndv_trestrft,ndv_trestrfx,ndv_trestlft,ndv_trestlfx,ndv_tractrhd,ndv_tractrhx,ndv_tractlhd,ndv_tractlhx,ndv_rigdneck,ndv_rigdnex,ndv_rigduprt,ndv_rigduprx,ndv_rigduplf,ndv_rigduplx,ndv_rigdlort,ndv_rigdlorx,ndv_rigdlolf,ndv_rigdlolx,ndv_tapsrt,ndv_tapsrtx,ndv_tapslf,ndv_tapslfx,ndv_handmovr,ndv_handmvrx,ndv_handmovl,ndv_handmvlx,ndv_handaltr,ndv_handatrx,ndv_handaltl,ndv_handatlx,ndv_legrt,ndv_legrtx,ndv_leglf,ndv_leglfx,ndv_arising,ndv_arisingx,ndv_posture,ndv_posturex,ndv_gait,ndv_gaitx,ndv_posstab,ndv_posstabx,ndv_bradykin,ndv_bradykix,ndv_memory,ndv_orient,ndv_judgment,ndv_commun,ndv_homehobb,ndv_perscare,ndv_cdrsum,ndv_cdrglob,ndv_comport,ndv_cdrlang,ndv_npiqinf,ndv_npiqinfx,ndv_del,ndv_delsev,ndv_hall,ndv_hallsev,ndv_agit,ndv_agitsev,ndv_depd,ndv_depdsev,ndv_anx,ndv_anxsev,ndv_elat,ndv_elatsev,ndv_apa,ndv_apasev,ndv_disn,ndv_disnsev,ndv_irr,ndv_irrsev,ndv_mot,ndv_motsev,ndv_nite,ndv_nitesev,ndv_app,ndv_appsev,ndv_nogds,ndv_satis,ndv_dropact,ndv_empty,ndv_bored,ndv_spirits,ndv_afraid,ndv_happy,ndv_helpless,ndv_stayhome,ndv_memprob,ndv_wondrful,ndv_wrthless,ndv_energy,ndv_hopeless,ndv_better,ndv_naccgds,ndv_bills,ndv_taxes,ndv_shopping,ndv_games,ndv_stove,ndv_mealprep,ndv_events,ndv_payattn,ndv_remdates,ndv_travel,ndv_naccnrex,ndv_normexam,ndv_focldef,ndv_gaitdis,ndv_eyemove,ndv_parksign,ndv_resttrl,ndv_resttrr,ndv_slowingl,ndv_slowingr,ndv_rigidl,ndv_rigidr,ndv_brady,ndv_parkgait,ndv_postinst,ndv_cvdsigns,ndv_cortdef,ndv_sivdfind,ndv_cvdmotl,ndv_cvdmotr,ndv_cortvisl,ndv_cortvisr,ndv_somatl,ndv_somatr,ndv_postcort,ndv_pspcbs,ndv_eyepsp,ndv_dyspsp,ndv_axialpsp,ndv_gaitpsp,ndv_apraxsp,ndv_apraxl,ndv_apraxr,ndv_cortsenl,ndv_cortsenr,ndv_ataxl,ndv_ataxr,ndv_alienlml,ndv_alienlmr,ndv_dystonl,ndv_dystonr,ndv_myocllt,ndv_myoclrt,ndv_alsfind,ndv_gaitnph,ndv_othneur,ndv_othneurx,ndv_normal,ndv_b9chg,ndv_decsub,ndv_decin,ndv_decclin,ndv_decclcog,ndv_cogmem,ndv_cogori,ndv_cogjudg,ndv_coglang,ndv_cogvis,ndv_cogattn,ndv_cogfluc,ndv_cogflago,ndv_cogothr,ndv_cogothrx,ndv_nacccogf,ndv_nacccgfx,ndv_cogmode,ndv_cogmodex,ndv_decage,ndv_decclbe,ndv_beapathy,ndv_bedep,ndv_bevhall,ndv_bevwell,ndv_bevhago,ndv_beahall,ndv_bedel,ndv_bedisin,ndv_beirrit,ndv_beagit,ndv_beperch,ndv_berem,ndv_beremago,ndv_beanx,ndv_beothr,ndv_beothrx,ndv_naccbehf,ndv_naccbefx,ndv_bemode,ndv_bemodex,ndv_beage,ndv_decclmot,ndv_mogait,ndv_mofalls,ndv_motrem,ndv_moslow,ndv_naccmotf,ndv_momode,ndv_momodex,ndv_momopark,ndv_parkage,ndv_momoals,ndv_alsage,ndv_moage,ndv_course,ndv_frstchg,ndv_lbdeval,ndv_ftldeval,ndv_mmsecomp,ndv_mmseloc,ndv_mmselan,ndv_mmselanx,ndv_mmsevis,ndv_mmsehear,ndv_mmseorda,ndv_mmseorlo,ndv_pentagon,ndv_naccmmse,ndv_npsycloc,ndv_npsylan,ndv_npsylanx,ndv_logimo,ndv_logiday,ndv_logiyr,ndv_logiprev,ndv_logimem,ndv_memunits,ndv_memtime,ndv_udsbentc,ndv_udsbentd,ndv_udsbenrs,ndv_digif,ndv_digiflen,ndv_digib,ndv_digiblen,ndv_animals,ndv_veg,ndv_traila,ndv_trailarr,ndv_trailali,ndv_trailb,ndv_trailbrr,ndv_trailbli,ndv_wais,ndv_boston,ndv_udsverfc,ndv_udsverfn,ndv_udsvernf,ndv_udsverlc,ndv_udsverlr,ndv_udsverln,ndv_udsvertn,ndv_udsverte,ndv_udsverti,ndv_cogstat,ndv_naccc1,ndv_mocacomp,ndv_mocareas,ndv_mocaloc,ndv_mocalan,ndv_mocalanx,ndv_mocavis,ndv_mocahear,ndv_mocatots,ndv_mocatrai,ndv_mocacube,ndv_mocacloc,ndv_mocaclon,ndv_mocacloh,ndv_mocanami,ndv_mocaregi,ndv_mocadigi,ndv_mocalett,ndv_mocaser7,ndv_mocarepe,ndv_mocaflue,ndv_mocaabst,ndv_mocarecn,ndv_mocarecc,ndv_mocarecr,ndv_mocaordt,ndv_mocaormo,ndv_mocaoryr,ndv_mocaordy,ndv_mocaorpl,ndv_mocaorct,ndv_craftvrs,ndv_crafturs,ndv_digforct,ndv_digforsl,ndv_digbacct,ndv_digbacls,ndv_craftdvr,ndv_craftdre,ndv_craftdti,ndv_craftcue,ndv_minttots,ndv_minttotw,ndv_mintscng,ndv_mintscnc,ndv_mintpcng,ndv_mintpcnc,ndv_naccc2,ndv_whodiddx,ndv_dxmethod,ndv_normcog,ndv_demented,ndv_amndem,ndv_pca,ndv_naccppa,ndv_naccppag,ndv_naccppme,ndv_naccbvft,ndv_nacclbds,ndv_namndem,ndv_nacctmci,ndv_naccmcil,ndv_naccmcia,ndv_naccmcie,ndv_naccmciv,ndv_naccmcii,ndv_impnomci,ndv_amylpet,ndv_amylcsf,ndv_fdgad,ndv_hippatr,ndv_taupetad,ndv_csftau,ndv_fdgftld,ndv_tpetftld,ndv_mrftld,ndv_datscan,ndv_othbiom,ndv_othbiomx,ndv_imaglinf,ndv_imaglac,ndv_imagmach,ndv_imagmich,ndv_imagmwmh,ndv_imagewmh,ndv_othmut,ndv_othmutx,ndv_naccalzd,ndv_naccalzp,ndv_probad,ndv_probadif,ndv_possad,ndv_possadif,ndv_nacclbde,ndv_nacclbdp,ndv_park,ndv_msa,ndv_msaif,ndv_psp,ndv_pspif,ndv_cort,ndv_cortif,ndv_ftldmo,ndv_ftldmoif,ndv_ftldnos,ndv_ftldnoif,ndv_ftd,ndv_ftdif,ndv_ppaph,ndv_ppaphif,ndv_ftldsubt,ndv_ftldsubx,ndv_cvd,ndv_cvdif,ndv_prevstk,ndv_strokdec,ndv_stkimag,ndv_infnetw,ndv_infwmh,ndv_vasc,ndv_vascif,ndv_vascps,ndv_vascpsif,ndv_stroke,ndv_strokif,ndv_esstrem,ndv_esstreif,ndv_downs,ndv_downsif,ndv_hunt,ndv_huntif,ndv_prion,ndv_prionif,ndv_brninj,ndv_brninjif,ndv_brnincte,ndv_hyceph,ndv_hycephif,ndv_epilep,ndv_epilepif,ndv_neop,ndv_neopif,ndv_neopstat,ndv_hiv,ndv_hivif,ndv_othcog,ndv_othcogif,ndv_othcogx,ndv_dep,ndv_depif,ndv_deptreat,ndv_bipoldx,ndv_bipoldif,ndv_schizop,ndv_schizoif,ndv_anxiet,ndv_anxietif,ndv_delir,ndv_delirif,ndv_ptsddx,ndv_ptsddxif,ndv_othpsy,ndv_othpsyif,ndv_othpsyx,ndv_alcdem,ndv_alcdemif,ndv_alcabuse,ndv_impsub,ndv_impsubif,ndv_dysill,ndv_dysillif,ndv_meds,ndv_medsif,ndv_demun,ndv_demunif,ndv_cogoth,ndv_cogothif,ndv_cogothx,ndv_cogoth2,ndv_cogoth2f,ndv_cogoth2x,ndv_cogoth3,ndv_cogoth3f,ndv_cogoth3x,ndv_naccetpr,ndv_naccadmu,ndv_naccftdm,ndv_cancer,ndv_cancsite,ndv_diabet,ndv_myoinf,ndv_conghrt,ndv_afibrill,ndv_hypert,ndv_angina,ndv_hypchol,ndv_vb12def,ndv_thydis,ndv_arth,ndv_artype,ndv_artypex,ndv_artupex,ndv_artloex,ndv_artspin,ndv_artunkn,ndv_urineinc,ndv_bowlinc,ndv_sleepap,ndv_remdis,ndv_hyposom,ndv_sleepoth,ndv_sleepotx,ndv_angiocp,ndv_angiopci,ndv_pacemake,ndv_hvalve,ndv_antienc,ndv_antiencx,ndv_othcond,ndv_othcondx,ndv_naccageb,ndv_naccnihr,ndv_naccnorm,ndv_naccidem,ndv_naccage,ndv_naccaaas,ndv_naccaanx,ndv_naccac,ndv_naccacei,ndv_naccadep,ndv_naccadmd,ndv_naccahtn,ndv_naccamd,ndv_naccangi,ndv_naccapsy,ndv_naccbeta,ndv_naccccbs,ndv_naccdbmd,ndv_naccdiur,ndv_naccemd,ndv_naccepmd,ndv_nacchtnc,ndv_nacclipl,ndv_naccnsd,ndv_naccpdmd,ndv_naccvasd,ndv_naccbmi,ndv_naccudsd,ndv_naccdied,ndv_naccmod,ndv_naccyod,ndv_naccautp,ndv_naccactv,ndv_naccnovs,ndv_naccdsmo,ndv_naccdsdy,ndv_naccdsyr,ndv_naccnurp,ndv_naccnrmo,ndv_naccnrdy,ndv_naccnryr,ndv_naccftd,ndv_naccmdss,ndv_naccpaff,ndv_nacclbdm,ndv_naccacsf,ndv_naccpcsf,ndv_nacctcsf,ndv_naccapoe,ndv_naccne4s,ndv_naccmri,ndv_naccnmri,ndv_adgcgwas,ndv_adgcrnd,ndv_adspseq,ndv_adspwes,ndv_adspwgs,ndv_naccdbgp,ndv_naccncrd,ndv_naccngds,ndv_npformver,ndv_npsex,ndv_nppmih,ndv_npfix,ndv_npfixx,ndv_npwbrwt,ndv_npwbrf,ndv_naccbrnn,ndv_npgrcca,ndv_npgrla,ndv_npgrha,ndv_npgrsnh,ndv_npgrlch,ndv_naccavas,ndv_nptan,ndv_nptanx,ndv_npaban,ndv_npabanx,ndv_npasan,ndv_npasanx,ndv_nptdpan,ndv_nptdpanx,ndv_nphismb,ndv_nphisg,ndv_nphisss,ndv_nphist,ndv_nphiso,ndv_nphisox,ndv_npthal,ndv_naccbraa,ndv_naccneur,ndv_npadnc,ndv_naccdiff,ndv_naccvasc,ndv_naccamy,ndv_nplinf,ndv_nplac,ndv_npinf,ndv_npinf1a,ndv_npinf1b,ndv_npinf1d,ndv_npinf1f,ndv_npinf2a,ndv_npinf2b,ndv_npinf2d,ndv_npinf2f,ndv_npinf3a,ndv_npinf3b,ndv_npinf3d,ndv_npinf3f,ndv_npinf4a,ndv_npinf4b,ndv_npinf4d,ndv_npinf4f,ndv_naccinf,ndv_nphem,ndv_nphemo,ndv_nphemo1,ndv_nphemo2,ndv_nphemo3,ndv_npmicro,ndv_npold,ndv_npold1,ndv_npold2,ndv_npold3,ndv_npold4,ndv_naccmicr,ndv_npoldd,ndv_npoldd1,ndv_npoldd2,ndv_npoldd3,ndv_npoldd4,ndv_nacchem,ndv_naccarte,ndv_npwmr,ndv_nppath,ndv_naccnec,ndv_nppath2,ndv_nppath3,ndv_nppath4,ndv_nppath5,ndv_nppath6,ndv_nppath7,ndv_nppath8,ndv_nppath9,ndv_nppath10,ndv_nppath11,ndv_nppatho,ndv_nppathox,ndv_npart,ndv_npoang,ndv_nacclewy,ndv_nplbod,ndv_npnloss,ndv_nphipscl,ndv_npscl,ndv_npftdtau,ndv_naccpick,ndv_npftdt2,ndv_nacccbd,ndv_naccprog,ndv_npftdt5,ndv_npftdt6,ndv_npftdt7,ndv_npftdt8,ndv_npftdt9,ndv_npftdt10,ndv_npfront,ndv_nptau,ndv_npftd,ndv_npftdtdp,ndv_npalsmnd,ndv_npoftd,ndv_npoftd1,ndv_npoftd2,ndv_npoftd3,ndv_npoftd4,ndv_npoftd5,ndv_npftdno,ndv_npftdspc,ndv_nptdpa,ndv_nptdpb,ndv_nptdpc,ndv_nptdpd,ndv_nptdpe,ndv_nppdxa,ndv_nppdxb,ndv_naccprio,ndv_nppdxd,ndv_nppdxe,ndv_nppdxf,ndv_nppdxg,ndv_nppdxh,ndv_nppdxi,ndv_nppdxj,ndv_nppdxk,ndv_nppdxl,ndv_nppdxm,ndv_nppdxn,ndv_naccdown,ndv_nppdxp,ndv_nppdxq,ndv_naccothp,ndv_naccwri1,ndv_naccwri2,ndv_naccwri3,ndv_naccbnkf,ndv_npbnkb,ndv_naccform,ndv_naccpara,ndv_nacccsfp,ndv_npbnkf,ndv_npfaut,ndv_npfaut1,ndv_npfaut2,ndv_npfaut3,ndv_npfaut4,ndv_naccdage,ndv_naccint,ndv_npnit,ndv_npcerad,ndv_npadrda,ndv_npocrit,ndv_npvoth,ndv_nplewycs,ndv_npgene,ndv_npfhspec,ndv_nptauhap,ndv_npprnp,ndv_npchrom,ndv_nppnorm,ndv_npcnorm,ndv_nppadp,ndv_npcadp,ndv_nppad,ndv_npcad,ndv_npplewy,ndv_npclewy,ndv_nppvasc,ndv_npcvasc,ndv_nppftld,ndv_npcftld,ndv_npphipp,ndv_npchipp,ndv_nppprion,ndv_npcprion,ndv_nppoth1,ndv_npcoth1,ndv_npoth1x,ndv_nppoth2,ndv_npcoth2,ndv_npoth2x,ndv_nppoth3,ndv_npcoth3,ndv_npoth3x,ndv_a2not,ndv_a2sub,ndv_a3not,ndv_a3sub,ndv_a4not,ndv_a4sub,ndv_b1not,ndv_b1sub,ndv_b5not,ndv_b5sub,ndv_b6not,ndv_b6sub,ndv_b7not,ndv_b7sub,ndv_clssub,ndv_b2not,ndv_b2sub,ndv_b3not,ndv_b3sub,ndv_b8not,ndv_b8sub,ndv_ftda3afr,ndv_ftda3afs,ndv_ftdc4fr,ndv_ftdc4fs,ndv_ftdc5fr,ndv_ftdc5fs,ndv_ftdc6fr,ndv_ftdc6fs,ndv_langa1,ndv_langa2,ndv_langa3,ndv_langa3a,ndv_langa4,ndv_langa5,ndv_langb1,ndv_langb3f,ndv_langb4,ndv_langb5,ndv_langb6,ndv_langb7,ndv_langb8,ndv_langb9,ndv_langb9f,ndv_langc1,ndv_langc1f,ndv_langc2,ndv_langc2f,ndv_langc3f,ndv_langc4f,ndv_langc5f,ndv_langc6f,ndv_langcls,ndv_langd1,ndv_langd2,ndv_lange2f,ndv_lange3f,ndv_langt1,ndv_a2comm,ndv_a3comm,ndv_a4comm,ndv_b1comm,ndv_b2comm,ndv_b3comm,ndv_b5comm,ndv_b6comm,ndv_b7comm,ndv_b8comm,ndv_telcog,ndv_telill,ndv_telhome,ndv_telrefu,ndv_telothr,ndv_telothrx,ndv_telmile,ndv_telinper,ndv_learned,ndv_prestat,ndv_prespart,ndv_source,ndv_sourcenw,ndv_afffamm,ndv_a3chg,ndv_parchg,ndv_nwinfpar,ndv_mommob,ndv_momyob,ndv_momdage,ndv_mommoe,ndv_momageo,ndv_momliv,ndv_momyod,ndv_momonset,ndv_momage,ndv_dadmob,ndv_dadyob,ndv_daddage,ndv_dadmoe,ndv_dadageo,ndv_dadliv,ndv_dadyod,ndv_daddem,ndv_dadonset,ndv_dadage,ndv_sibchg,ndv_twin,ndv_twintype,ndv_sibs,ndv_nwinfsib,ndv_sib1mob,ndv_sib1yob,ndv_sib1agd,ndv_sib1neu,ndv_sib1pdx,ndv_sib1moe,ndv_sib1ago,ndv_sib2mob,ndv_sib2yob,ndv_sib2agd,ndv_sib2neu,ndv_sib2pdx,ndv_sib2moe,ndv_sib2ago,ndv_sib3mob,ndv_sib3yob,ndv_sib3agd,ndv_sib3neu,ndv_sib3pdx,ndv_sib3moe,ndv_sib3ago,ndv_sib4mob,ndv_sib4yob,ndv_sib4agd,ndv_sib4neu,ndv_sib4pdx,ndv_sib4moe,ndv_sib4ago,ndv_sib5mob,ndv_sib5yob,ndv_sib5agd,ndv_sib5neu,ndv_sib5pdx,ndv_sib5moe,ndv_sib5ago,ndv_sib6mob,ndv_sib6yob,ndv_sib6agd,ndv_sib6neu,ndv_sib6pdx,ndv_sib6moe,ndv_sib6ago,ndv_sib7mob,ndv_sib7yob,ndv_sib7agd,ndv_sib7neu,ndv_sib7pdx,ndv_sib7moe,ndv_sib7ago,ndv_sib8mob,ndv_sib8yob,ndv_sib8agd,ndv_sib8neu,ndv_sib8pdx,ndv_sib8moe,ndv_sib8ago,ndv_sib9mob,ndv_sib9yob,ndv_sib9agd,ndv_sib9neu,ndv_sib9pdx,ndv_sib9moe,ndv_sib9ago,ndv_sib10mob,ndv_sib10yob,ndv_sib10agd,ndv_sib10neu,ndv_sib10pdx,ndv_sib10moe,ndv_sib10ago,ndv_sib11mob,ndv_sib11yob,ndv_sib11agd,ndv_sib11neu,ndv_sib11pdx,ndv_sib11moe,ndv_sib11ago,ndv_sib12mob,ndv_sib12yob,ndv_sib12agd,ndv_sib12neu,ndv_sib12pdx,ndv_sib12moe,ndv_sib12ago,ndv_sib13mob,ndv_sib13yob,ndv_sib13agd,ndv_sib13neu,ndv_sib13pdx,ndv_sib13moe,ndv_sib13ago,ndv_sib14mob,ndv_sib14yob,ndv_sib14agd,ndv_sib14neu,ndv_sib14pdx,ndv_sib14moe,ndv_sib14ago,ndv_sib15mob,ndv_sib15yob,ndv_sib15agd,ndv_sib15neu,ndv_sib15pdx,ndv_sib15moe,ndv_sib15ago,ndv_sib16mob,ndv_sib16yob,ndv_sib16agd,ndv_sib16neu,ndv_sib16pdx,ndv_sib16moe,ndv_sib16ago,ndv_sib17mob,ndv_sib17yob,ndv_sib17agd,ndv_sib17neu,ndv_sib17pdx,ndv_sib17moe,ndv_sib17ago,ndv_sib18mob,ndv_sib18yob,ndv_sib18agd,ndv_sib18neu,ndv_sib18pdx,ndv_sib18moe,ndv_sib18ago,ndv_sib19mob,ndv_sib19yob,ndv_sib19agd,ndv_sib19neu,ndv_sib19pdx,ndv_sib19moe,ndv_sib19ago,ndv_sib20mob,ndv_sib20yob,ndv_sib20agd,ndv_sib20neu,ndv_sib20pdx,ndv_sib20moe,ndv_sib20ago,ndv_sib1liv,ndv_sib1yod,ndv_sibsdem,ndv_sib1dem,ndv_sib1ons,ndv_sib1age,ndv_sib2liv,ndv_sib2yod,ndv_sib2dem,ndv_sib2ons,ndv_sib2age,ndv_sib3liv,ndv_sib3yod,ndv_sib3dem,ndv_sib3ons,ndv_sib3age,ndv_sib4liv,ndv_sib4yod,ndv_sib4dem,ndv_sib4ons,ndv_sib4age,ndv_sib5liv,ndv_sib5yod,ndv_sib5dem,ndv_sib5ons,ndv_sib5age,ndv_sib6liv,ndv_sib6yod,ndv_sib6dem,ndv_sib6ons,ndv_sib6age,ndv_sib7liv,ndv_sib7yod,ndv_sib7dem,ndv_sib7ons,ndv_sib8liv,ndv_sib8yod,ndv_sib8dem,ndv_sib8ons,ndv_sib9liv,ndv_sib9yod,ndv_sib9dem,ndv_sib9ons,ndv_sib10liv,ndv_sib10yod,ndv_sib10dem,ndv_sib10ons,ndv_sib11liv,ndv_sib11yod,ndv_sib11dem,ndv_sib11ons,ndv_sib12liv,ndv_sib12yod,ndv_sib12dem,ndv_sib12ons,ndv_sib13liv,ndv_sib13yod,ndv_sib13dem,ndv_sib13ons,ndv_sib14liv,ndv_sib14yod,ndv_sib14dem,ndv_sib14ons,ndv_sib15liv,ndv_sib15yod,ndv_sib15dem,ndv_sib15ons,ndv_sib16liv,ndv_sib16yod,ndv_sib16dem,ndv_sib16ons,ndv_sib17liv,ndv_sib17yod,ndv_sib17dem,ndv_sib17ons,ndv_sib18liv,ndv_sib18yod,ndv_sib18dem,ndv_sib18ons,ndv_sib19liv,ndv_sib19yod,ndv_sib19dem,ndv_sib19ons,ndv_sib20liv,ndv_sib20yod,ndv_sib20dem,ndv_sib20ons,ndv_kidchg,ndv_kids,ndv_nwinfkid,ndv_kid1mob,ndv_kid1yob,ndv_kid1agd,ndv_kid1neu,ndv_kid1pdx,ndv_kid1moe,ndv_kid1ago,ndv_kid2mob,ndv_kid2yob,ndv_kid2agd,ndv_kid2neu,ndv_kid2pdx,ndv_kid2moe,ndv_kid2ago,ndv_kid3mob,ndv_kid3yob,ndv_kid3agd,ndv_kid3neu,ndv_kid3pdx,ndv_kid3moe,ndv_kid3ago,ndv_kid4mob,ndv_kid4yob,ndv_kid4agd,ndv_kid4neu,ndv_kid4pdx,ndv_kid4moe,ndv_kid4ago,ndv_kid5mob,ndv_kid5yob,ndv_kid5agd,ndv_kid5neu,ndv_kid5pdx,ndv_kid5moe,ndv_kid5ago,ndv_kid6mob,ndv_kid6yob,ndv_kid6agd,ndv_kid6neu,ndv_kid6pdx,ndv_kid6moe,ndv_kid6ago,ndv_kid7mob,ndv_kid7yob,ndv_kid7agd,ndv_kid7neu,ndv_kid7pdx,ndv_kid7moe,ndv_kid7ago,ndv_kid8mob,ndv_kid8yob,ndv_kid8agd,ndv_kid8neu,ndv_kid8pdx,ndv_kid8moe,ndv_kid8ago,ndv_kid9mob,ndv_kid9yob,ndv_kid9agd,ndv_kid9neu,ndv_kid9pdx,ndv_kid9moe,ndv_kid9ago,ndv_kid10mob,ndv_kid10yob,ndv_kid10agd,ndv_kid10neu,ndv_kid10pdx,ndv_kid10moe,ndv_kid10ago,ndv_kid11mob,ndv_kid11yob,ndv_kid11agd,ndv_kid11neu,ndv_kid11pdx,ndv_kid11moe,ndv_kid11ago,ndv_kid12mob,ndv_kid12yob,ndv_kid12agd,ndv_kid12neu,ndv_kid12pdx,ndv_kid12moe,ndv_kid12ago,ndv_kid13mob,ndv_kid13yob,ndv_kid13agd,ndv_kid13neu,ndv_kid13pdx,ndv_kid13moe,ndv_kid13ago,ndv_kid14mob,ndv_kid14yob,ndv_kid14agd,ndv_kid14neu,ndv_kid14pdx,ndv_kid14moe,ndv_kid14ago,ndv_kid15mob,ndv_kid15yob,ndv_kid15agd,ndv_kid15neu,ndv_kid15pdx,ndv_kid15moe,ndv_kid15ago,ndv_kid1liv,ndv_kid1yod,ndv_kidsdem,ndv_kid1dem,ndv_kid1ons,ndv_kid1age,ndv_kid2liv,ndv_kid2yod,ndv_kid2dem,ndv_kid2ons,ndv_kid2age,ndv_kid3liv,ndv_kid3yod,ndv_kid3dem,ndv_kid3ons,ndv_kid3age,ndv_kid4liv,ndv_kid4yod,ndv_kid4dem,ndv_kid4ons,ndv_kid4age,ndv_kid5liv,ndv_kid5yod,ndv_kid5dem,ndv_kid5ons,ndv_kid5age,ndv_kid6liv,ndv_kid6yod,ndv_kid6dem,ndv_kid6ons,ndv_kid6age,ndv_kid7liv,ndv_kid7yod,ndv_kid7dem,ndv_kid7ons,ndv_kid8liv,ndv_kid8yod,ndv_kid8dem,ndv_kid8ons,ndv_kid9liv,ndv_kid9yod,ndv_kid9dem,ndv_kid9ons,ndv_kid10liv,ndv_kid10yod,ndv_kid10dem,ndv_kid10ons,ndv_kid11liv,ndv_kid11yod,ndv_kid11dem,ndv_kid11ons,ndv_kid12liv,ndv_kid12yod,ndv_kid12dem,ndv_kid12ons,ndv_kid13liv,ndv_kid13yod,ndv_kid13dem,ndv_kid13ons,ndv_kid14liv,ndv_kid14yod,ndv_kid14dem,ndv_kid14ons,ndv_kid15liv,ndv_kid15yod,ndv_kid15dem,ndv_kid15ons,ndv_relchg,ndv_relsdem,ndv_rel1yob,ndv_rel1liv,ndv_rel1yod,ndv_rel1ons,ndv_rel1age,ndv_rel2yob,ndv_rel2liv,ndv_rel2yod,ndv_rel2ons,ndv_rel2age,ndv_rel3yob,ndv_rel3liv,ndv_rel3yod,ndv_rel3ons,ndv_rel3age,ndv_rel4yob,ndv_rel4liv,ndv_rel4yod,ndv_rel4ons,ndv_rel4age,ndv_rel5yob,ndv_rel5liv,ndv_rel5yod,ndv_rel5ons,ndv_rel5age,ndv_rel6yob,ndv_rel6liv,ndv_rel6yod,ndv_rel6ons,ndv_rel6age,ndv_rel7yob,ndv_rel7liv,ndv_rel7yod,ndv_rel7ons,ndv_rel8yob,ndv_rel8liv,ndv_rel8yod,ndv_rel8ons,ndv_rel9yob,ndv_rel9liv,ndv_rel9yod,ndv_rel9ons,ndv_rel10yob,ndv_rel10liv,ndv_rel10yod,ndv_rel10ons,ndv_rel11yob,ndv_rel11liv,ndv_rel11yod,ndv_rel11ons,ndv_rel12yob,ndv_rel12liv,ndv_rel12yod,ndv_rel12ons,ndv_rel13yob,ndv_rel13liv,ndv_rel13yod,ndv_rel13ons,ndv_rel14yob,ndv_rel14liv,ndv_rel14yod,ndv_rel14ons,ndv_rel15yob,ndv_rel15liv,ndv_rel15yod,ndv_rel15ons,ndv_fadmuso,ndv_fadmut,ndv_fftdmuso,ndv_fftdmut,ndv_fothmuso,ndv_fothmut,ndv_fadmutx,ndv_fadmusox,ndv_fftdmutx,ndv_fftdmusx,ndv_fothmutx,ndv_fothmusx,ndv_cbothr,ndv_cbothrx,ndv_mmsereas,ndv_parkif,ndv_ctflm,ndv_ctdig,ndv_mri1flm,ndv_mri1dig,ndv_mri2flm,ndv_mri2dig,ndv_mri3flm,ndv_mri3dig,ndv_mrispflm,ndv_mrispdig,ndv_spectflm,ndv_spectdig,ndv_petflm,ndv_petdig,ndv_dna,ndv_csfantem,ndv_serum,ndv_apoe,ndv_reason,ndv_reasonx,ndv_refer,ndv_referx,ndv_refersc,ndv_livsit,ndv_livsitx,ndv_livsitua,ndv_momdem,ndv_momneur,ndv_momprdx,ndv_dadneur,ndv_dadprdx,ndv_strok1yr,ndv_strokyr,ndv_strok2yr,ndv_strok3yr,ndv_strok4yr,ndv_strok5yr,ndv_strok6yr,ndv_tia1yr,ndv_tiayear,ndv_tia2yr,ndv_tia3yr,ndv_tia4yr,ndv_tia5yr,ndv_tia6yr,ndv_gds,ndv_cogfrst,ndv_cogfpred,ndv_cogfrstx,ndv_cogfprex,ndv_befrst,ndv_befpred,ndv_befrstx,ndv_befpredx,ndv_mofrst,ndv_pnaph,ndv_semdeman,ndv_semdemag,ndv_ppaothr,ndv_mciamem,ndv_mciaplus,ndv_mcinon1,ndv_mcinon2,ndv_mciaplan,ndv_mcin1lan,ndv_mcin2lan,ndv_mciapatt,ndv_mcin1att,ndv_mcin2att,ndv_mciapex,ndv_mcin1ex,ndv_mcin2ex,ndv_mciapvis,ndv_mcin1vis,ndv_mcin2vis,ndv_alzdis,ndv_lbdsyn,ndv_ppasyn,ndv_ppasynt,ndv_alzdisif,ndv_dlb,ndv_lbdis,ndv_dlbif,ndv_lbdif,ndv_nwinfmut,ndv_admut,ndv_ftdsyn,ndv_ftldmut,ndv_mmse,nacc_derived_values_complete"  # noqa: E501

participantIVP = ""  # noqa: E501


class TestC2Cutover(unittest.TestCase):

    @unittest.skip("This test requires some mock data before it'll pass.")
    def test_use_c1s_before_cutover(self):
        """Use C1S before the 2017-10-23 cutover."""
        fp = io.StringIO('\n'.join([headers, participantIVP]))
        out = io.StringIO()
        err = io.StringIO()

        options = redcap2nacc.parse_args([])
        redcap2nacc.convert(fp, options, out, err)

        e = read_all(err)
        out.close(), err.close(), fp.close()

        self.assertTrue(len(e) > 0, "Expected error messages")

    @unittest.skip("This test is just a placeholder.")
    def test_use_c2_on_cutover(self):
        """Use C2 on the 2017-10-23 cutover."""
        self.assertFalse(True)


def read_all(reader):
    reader.seek(0)
    return reader.read()


if __name__ == "__main__":
    unittest.main()
